<?php
	// $Id
	require_once dirname(__FILE__).'/'.'eDUtilities.php';

	/**
	 * eDDataFile - Data File Generator
	 *
     * This script allows the easy creation of flat data files from database
     * records. Data to be written in the file are provided as an array of
     * associative arrays. One associative arrays is made available per record
     * retrieved, keys being the name of the database fields (eDQuery 
     * principle).
     * From that list of records, a filter can be provided to indicate which
     * fields should be extracted in the file and in which order. Other fields
     * are ignored. Name and path of the file can be specified as well as the 
     * field and line separator.
     * Some example of use:
     * - Creation of Comma Separated Values (CSV) files from database record
     * allowing to load them in a worksheet application such as MS EXCEL.
     * - Creation of flat files used as inputs by some external programs for
     * further processing
     * - ...
	 *
	 * @author		Bertrand Potier <info@edreamers.org>
	 * @package		eDFramework
	 * @see			eDUtilities
	 * @link		http://eDreamers.org
	 */
	class eDDataFile extends eDUtilities {
	
    /**
	 * eDDataFile::$str_filename
	 *
	 * Name of the file to create
	 */
	 var $str_filename = 'file.dat';

    /**
	 * eDDataFile::$str_filepath
	 *
	 * Path to the location on the filesystem where to create the data file
	 */
	 var $str_filepath = null;

    /**
	 * eDDataFile::$str_separator
	 *
	 * Character or string to be used to separate record fields
	 */
	 var $str_separator = ',';

    /**
	 * eDDataFile::$str_endofline
	 *
	 * Character to use to terminate a record line

	 */
	 var $str_endofline = "\n";

    /**
	 * eDDataFile::$arr_records
	 *
	 * The array of records
	 */
	 var $arr_records = array();

    /**
	 * eDDataFile::$arr_data
	 *
	 * The array of data being either a copy of the records or, if filters are 
     * provided, the extract from the records of the only fields to be included
	 */
	 var $arr_data = array();

    /**
	 * eDDataFile::$int_fp
	 *
	 * The file pointer
	 */
	 var $int_fp = null;

    /**
	 * eDDataFile::$traceMode
	 *
	 * Trace mode, either screen or log
	 */
	 var $traceMode = 'log';

	/**
     * Class Constructor
     *
     * Today the constructor launches the construction and the execution of the query (extended mode) but
     * this can be moved away to stay under the responsability of the script calling eDDataFile
     *
     * @param		string		$dbName		name of the db against which the query will be launched
     * @param		array		$arr_from	array containing the list of table to query, ex: array('system', 'employee')
     * @param		array		$arr_select	array containing the list of fields to retrieve, ex: array('systemid', 'employeeid')
     * @param		array		$arr_where	array describing the where clause, ex: array('systemid' => "'1'") or array("systemid > '1'" => '') for clause not based on = operators
     * @param		string		$mode		only build the query in normal mode, build and run the query in extended mode
     * @param		string		$orderBy	name of the field(s) to use to order the results of the query
     * @param		boolean	    $debug		enter function in debug mode (enables outputs) or not
     */
    function eDDataFile(
        $arr_records,
        $arr_filters = null,
        $str_filename = 'file.dat',
        $str_filepath = null,
        $str_separator = ',',
        $str_endofline = "\n",
        $traceEnabled = false,
        $mode = 'extended',
        $int_fp = null
    ) {
        $this->setTrace( $traceEnabled, 'log', 'text', 'eDDataFile.log' );
        $this->trace(__FILE__.' :: __START__ :: Function eDDataFile Constructor', 'inline' );

        // Initialising Class variables
        $this->arr_records = $arr_records;
        $this->str_filename = $str_filename;
        $this->str_separator = $str_separator;
        $this->str_endofline = $str_endofline;
        
        // Testing filepath, using default value if not available
        if ( ! $str_filepath ) { 
            $str_filepath = dirname(__FILE__).'\\';
            $this->trace(__FILE__.' :: Line '.__LINE__.' :: Watchpoint 2 :: '.$str_filepath, 'inline' );
        }
        $this->str_filepath = $str_filepath;

        // Testing file pointer, if null then mode is overwritten to 'basic'
        if ( $int_fp == null ) {
            $mode = 'basic';
            $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function eDDataFile - no valid file pointer, mode is set to basic: Provide a file pointer or make an explicit call to the createFile() function!', 'inline' );
        } else {
            $this->int_fp = $int_fp;
        }
        

        switch($mode) {
            case 'basic':
                $int_fp = $this->createFile( $str_filename, $str_filepath );
                if ( $arr_filters ) { 
                    $this->trace(__FILE__.' :: Line '.__LINE__.' :: WATCH - Filters activated!', 'inline' );
                    $arr_data = $this->filterRecords( $arr_records, $arr_filters ); 
                } else {
                    $arr_data = $arr_records;
                }
            case 'advanced':
            case 'extended':
                $this->fillData( $arr_data, $int_fp, $str_separator, $str_endofline );
                break;
            default:
                break;
        }
    }



	/**
	 * Method :: filterRecords
	 *
	 * Filter an array of records based on a filter definition array
	 *
	 * @since eDDataFile v1.0
	 */
    function filterRecords(
        $arr_records,
        $arr_filters
    ) {
        $arr_ = array();

        $n = sizeof( $arr_filters );

        foreach( $arr_records as $key => $record ) {
            for ( $i = 0; $i < $n; $i++ ) {
                $arr_[$i] = $record[$arr_filters[$i]];
            }
            $arr_records[$key] = $arr_;
        }
        return $arr_records;
    }


	/**
	 * Method :: createFile
     *
     * Create the data file
	 *
     * { Description }
	 *
	 * Change Log
     *
     * @since eDDataFile v1.0
	 */
    function createFile(
        $str_filename = 'file.dat',
        $str_filepath = null
    ) {
        $int_fp = null;

        $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function createFile :: ('.$str_filepath.' - '.$str_filename.')', 'inline');
        if ( $int_fp = fopen( $str_filepath.$str_filename, "w" ) ) {
            $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function createFile :: SUCCESS - Creation of data file successful! ('.$str_filepath.' - '.$str_filename.')', 'inline');
            return $int_fp;
        } else {
            $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function createFile :: ERROR - Creation of data file failed! ('.$str_filepath.' - '.$str_filename.')', 'inline');
            return false;
        }
    }


    /**
     * Build the query
     *
     * Change Log
     * 14/08/2003   BPO First version
     * xx/xx/xxxx	WHO	SFxx#000000	Title
     */
    function fillData(
        $arr_records,
        $int_fp,
        $str_separator = ',',
        $str_endofline = "\n"
    ) {
        $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function fillData :: '.sizeof( $arr_records ).' record(s) to be written', 'inline');
        
        $str_ = ''; // temporary variable to store the data to be written in the file
        
        $k = sizeof($arr_records);
        $j = 0;
        foreach ( $arr_records as $key => $record ) {
            $n = sizeof( $record );
            $i = 0;
            foreach ( $record as $key2 => $value ) {
                $str_ .= $value;
                $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function fillData ::  WATCH - '.$key2.' - '.$n, 'inline');
                if ( ($i == $n-1) && ($j != $k-1) ) {
                    $str_ .= $str_endofline;
                } else {
                    $str_ .= $str_separator;
                }
                $i++;
            }
            $j++;
        }
        $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function fillData - the following data will be written: '.$str_, 'inline');

        // Writing data into the file
        if ( $int_bytes = fwrite( $int_fp, $str_ ) ) {
            $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function fillData - Successfully wrote '.$int_bytes.' bytes of data', 'inline');
        } else {
            $this->trace(__FILE__.' :: Line '.__LINE__.' :: Function fillData - ERROR: Failed writing data to file!', 'inline');
        }

    }


    /**
     * Method :: getData()
     *
     * Returns the array of data being written to the data file
     *
     * Change Log
     * 14/08/2003   BPO First version
     * xx/xx/xxxx	WHO	SFxx#000000	Title
     */
    function getData() {
        return $this->arr_data;
    }
}
?>